@@ -16,7 +16,7 @@ char PROG[20] = "abpoa";
 #define _bO BOLD UNDERLINE "O" NONE
 #define _bA BOLD UNDERLINE "A" NONE
 char DESCRIPTION[100] = _ba "daptive " _bb "anded " _bP "artial " _bO "rder " _bA "lignment";
-char VERSION[20] = "1.5.3";
+char VERSION[20] = "1.5.5";
 char CONTACT[30] = "yangao@ds.dfci.harvard.edu";
 
 const struct option abpoa_long_opt [] = {
@@ -39,18 +39,21 @@ const struct option abpoa_long_opt [] = {
     { "min-poa-win", 1, NULL, 'n' },
     { "progressive", 0, NULL, 'p'},
 
+    { "inc-path-score", 0, NULL, 'G'},
+    { "sort-by-len", 0, NULL, 'L'},
+    { "gap-on-right", 0, NULL, 'R' },
+    { "gap-at-end", 0, NULL, 'J' },
     { "use-qual-weight", 0, NULL, 'Q'},
     { "amino-acid", 0, NULL, 'c'},
     { "in-list", 0, NULL, 'l' },
     { "increment", 1, NULL, 'i' },
-    
 
     { "amb-strand", 0, NULL, 's' },
     { "output", 1, NULL, 'o' },
     { "result", 1, NULL, 'r' },
     { "out-pog", 1, NULL, 'g' },
     { "cons-algrm", 1, NULL, 'a'},
-    { "max-num-cons", 1, NULL, 'd', },
+    { "maxnum-cons", 1, NULL, 'd', },
     { "min-freq", 1, NULL, 'q', },
 
     { "help", 0, NULL, 'h' },
@@ -90,6 +93,11 @@ int abpoa_usage(void)
     err_printf("    -f --extra-f   FLOAT    second adaptive banding parameter [%.2f]\n", ABPOA_EXTRA_F);
     err_printf("                            the number of extra bases added on both sites of the band is\n");
     err_printf("                            b+f*L, where L is the length of the aligned sequence\n");
+    err_printf("  Heuristics for graph alignment: (**under development**)\n");
+    err_printf("    -G --inc-path-score     include log-scaled path score for graph alignment [False]\n");
+    err_printf("    -L --sort-by-len        sort input sequences by length in descending order [False]\n");
+    err_printf("    -R --gap-on-right       put indel on the right-most side of the alignment, default is left [False]\n");
+    err_printf("    -J --gap-at-end         always put indel at the end of the alignment if possible [False]\n");
     // err_printf("    -z --zdrop    INT       Z-drop score in extension alignment [-1]\n");
     // err_printf("                            set as <= 0 to disable Z-drop extension\n");
     // err_printf("    -e --bonus    INT       end bonus score in extension alignment [-1]\n");
@@ -119,11 +127,13 @@ int abpoa_usage(void)
     err_printf("                            - %d: graph in GFA format\n", ABPOA_OUT_GFA);
     err_printf("                            - %d: graph with consensus path in GFA format\n", ABPOA_OUT_CONS_GFA);
     err_printf("                            - %d: consensus in FASTQ format\n", ABPOA_OUT_CONS_FQ);
-    err_printf("    -a --cons-algrm INT     consensus algorithm [%d]\n", ABPOA_HB);
+    err_printf("    -a --cons-algrm  INT    consensus algorithm [%d]\n", ABPOA_HB);
     err_printf("                            - %d: heaviest bundling path in partial order graph\n", ABPOA_HB);
     err_printf("                            - %d: most frequent bases at each position\n", ABPOA_MF);
     err_printf("    -d --maxnum-cons INT    max. number of consensus sequence to generate [1]\n");
-    err_printf("    -q --min-freq  FLOAT    min. frequency of each consensus sequence (only effective when -d/--num-cons > 1) [%.2f]\n", MULTIP_MIN_FREQ);
+    err_printf("                            if >=2, sequences will be clustered based on MSA and consensus will be generated within each cluster\n");
+    err_printf("                            up to 10 is supported currently (experimental for >=2)\n");
+    err_printf("    -q --min-freq  FLOAT    min. frequency of each cluster (only effective when -d/--maxnum-cons >= 2) [%.2f]\n", MULTIP_MIN_FREQ);
     err_printf("    -g --out-pog    FILE    dump final alignment graph to FILE (.pdf/.png) [Null]\n\n");
 
     err_printf("    -h --help               print this help usage information\n");
@@ -141,9 +151,12 @@ int abpoa_main(char *file_fn, int is_list, abpoa_para_t *abpt){
     abpoa_t *ab = abpoa_init();
     if (is_list) { // input file list
         FILE *list_fp = fopen(file_fn, "r"); char read_fn[1024];
+        int batch_index = 1;
         while (fgets(read_fn, sizeof(read_fn), list_fp)) {
             read_fn[strlen(read_fn)-1] = '\0';
+            abpt->batch_index = batch_index;
             abpoa_msa1(ab, abpt, read_fn, stdout);
+            batch_index++;
         }
         fclose(list_fp);
     } else // input file
@@ -156,19 +169,23 @@ int abpoa_main(char *file_fn, int is_list, abpoa_para_t *abpt){
 
 int main(int argc, char **argv) {
     int c, m, in_list=0; char *s; abpoa_para_t *abpt = abpoa_init_para();
-    while ((c = getopt_long(argc, argv, "m:M:X:t:O:E:b:f:z:e:QSk:w:n:i:clpso:r:g:a:d:q:hvV:", abpoa_long_opt, NULL)) >= 0) {
+    while ((c = getopt_long(argc, argv, "m:M:X:t:O:E:b:f:z:e:GLRJQSk:w:n:i:clpso:r:g:a:d:q:hvV:", abpoa_long_opt, NULL)) >= 0) {
         switch(c)
         {
             case 'm': m = atoi(optarg);
                       if (m != ABPOA_GLOBAL_MODE && m != ABPOA_EXTEND_MODE && m != ABPOA_LOCAL_MODE) { 
-                          err_printf("Unknown alignment mode: %d.\n", m); return abpoa_usage();
+                          err_printf("Unknown alignment mode: %d.\n", m); return 1;
                       } abpt->align_mode=m; break;
             case 'M': abpt->match = atoi(optarg); break;
             case 'X': abpt->mismatch = atoi(optarg); break;
             case 't': abpt->use_score_matrix = 1; abpt->mat_fn = strdup(optarg); break;
-            case 'O': abpt->gap_open1 = strtol(optarg, &s, 10); if (*s == ',') abpt->gap_open2 = strtol(s+1, &s, 10); break;
-            case 'E': abpt->gap_ext1 = strtol(optarg, &s, 10); if (*s == ',') abpt->gap_ext2 = strtol(s+1, &s, 10); break;
+            case 'O': abpt->gap_open1 = strtol(optarg, &s, 10); if (*s == ',') abpt->gap_open2 = strtol(s+1, &s, 10); else abpt->gap_open2 = 0; break;
+            case 'E': abpt->gap_ext1 = strtol(optarg, &s, 10); if (*s == ',') abpt->gap_ext2 = strtol(s+1, &s, 10); else abpt->gap_ext2 = 0; break;
 
+            case 'G': abpt->inc_path_score = 1; break;
+            case 'L': abpt->sort_input_seq = 1; break;
+            case 'R': abpt->put_gap_on_right = 1; break;
+            case 'J': abpt->put_gap_at_end = 1; break;
             case 'b': abpt->wb = atoi(optarg); break;
             case 'f': abpt->wf = atof(optarg); break;
             case 'z': abpt->zdrop = atoi(optarg); break;
@@ -200,7 +217,12 @@ int main(int argc, char **argv) {
             case 'g': abpt->out_pog= strdup(optarg); break;
 
             case 'a': abpt->cons_algrm = atoi(optarg); break;
-            case 'd': abpt->max_n_cons = atoi(optarg); break; 
+            case 'd': abpt->max_n_cons = atoi(optarg);
+                      if (abpt->max_n_cons < 1 || abpt->max_n_cons > 10) {
+                          err_printf("Error: max number of consensus sequences should be 1~10.\n");
+                          return 1;
+                      }
+                      break; 
             case 'q': abpt->min_freq = atof(optarg); break;
 
             case 'h': return abpoa_usage();
