@@ -36,7 +36,7 @@
 #define ABPOA_OUT_CONS_FQ  5
 
 #define ABPOA_HB 0
-#define ABPOA_MC 1
+#define ABPOA_MF 1
 
 #define ABPOA_NONE_VERBOSE 0
 #define ABPOA_INFO_VERBOSE 1
@@ -68,6 +68,8 @@ typedef struct {
     int m; int *mat; char *mat_fn; // score matrix
     int use_score_matrix; // set _mat_ based on score matrix file, then _match_/_mismatch_ is not used.
     int match, max_mat, mismatch, min_mis, gap_open1, gap_open2, gap_ext1, gap_ext2; int inf_min;
+    int sort_input_seq; // sort input sequences by length, in descending order
+    int inc_path_score; // set mismatch/match score based on node weight, i.e., # covered reads * match/mismatch
     // minimizer seeding parameter
     int k, w, min_w;
     int wb; float wf; // extra band width
@@ -75,11 +77,13 @@ typedef struct {
     // int simd_flag; // available SIMD instruction
     // alignment mode
     uint8_t ret_cigar:1, rev_cigar:1, out_msa:1, out_cons:1, out_gfa:1, out_fq:1, use_read_ids:1, amb_strand:1;
-    uint8_t use_qv:1, disable_seeding:1, progressive_poa:1;
+    // sub_aln: reads align to subgraph, total read count is based on subgraph coverage, i.e., node.n_span_read, not total input read count
+    uint8_t sub_aln:1, use_qv:1, disable_seeding:1, progressive_poa:1, put_gap_on_right:1, put_gap_at_end:1; // put indel on the left-most side of the alignment: minimap2-like, or right-most: wfa2-like
     char *incr_fn, *out_pog;
     int align_mode, gap_mode, max_n_cons, cons_algrm; // consensus calling algorithm: 0: partial order graph, 1: majority voting
     double min_freq; // for multiploid data
     int verbose; // to control output msg
+    int batch_index; // index of current file in batch mode (for output header)
 
     // char LogTable65536[65536];
     // char bit_table16[65536];
@@ -87,9 +91,12 @@ typedef struct {
 
 typedef struct {
     int node_id;
-    int in_edge_n, in_edge_m, *in_id;
+    int in_edge_n, in_edge_m, *in_id; int *in_edge_weight; // in_edge_weight: for additional path score
     int out_edge_n, out_edge_m, *out_id; int *out_edge_weight; // out_edge_weight: edge-wise weight
-    int *read_weight, n_read, m_read; // read_weight: read-wise weight, valid when use_qv=1
+    // read_weight: read-wise weight, valid when use_qv=1
+    // n_read: # of reads covering this node
+    // n_span_read: # of reads spanning this node, including reads skipping this node
+    int *read_weight, n_read, m_read, n_span_read;
     uint64_t **read_ids; int read_ids_n; // for each edge
 
     int aligned_node_n, aligned_node_m, *aligned_node_id; // mismatch; aligned node will have same rank
