@@ -2,7 +2,7 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include "abpoa.h"
-#include "simd_abpoa_align.h"
+#include "abpoa_align_simd.h"
 #include "abpoa_align.h"
 #include "abpoa_seq.h"
 #include "abpoa_output.h"
@@ -85,6 +85,13 @@ void abpoa_set_mat_from_file(abpoa_para_t *abpt, char *mat_fn) {
 }
 
 void abpoa_set_gap_mode(abpoa_para_t *abpt) {
+    if (abpt->match < 0 || abpt->mismatch < 0 || abpt->gap_open1 < 0 || abpt->gap_open2 < 0 || abpt->gap_ext1 < 0 || abpt->gap_ext2 < 0) {
+        err_fatal(__func__, "Invalid negative scoring parameters: match=%d, mismatch=%d, gap_open1=%d, gap_open2=%d, gap_ext1=%d, gap_ext2=%d.",
+                  abpt->match, abpt->mismatch, abpt->gap_open1, abpt->gap_open2, abpt->gap_ext1, abpt->gap_ext2);
+    }
+    if (abpt->gap_ext1 == 0 && abpt->gap_ext2 == 0) {
+        err_fatal(__func__, "Invalid gap extension parameters, expect at least one postive: gap_ext1=%d, gap_ext2=%d.", abpt->gap_ext1, abpt->gap_ext2);
+    }
     if (abpt->gap_open1 == 0) abpt->gap_mode = ABPOA_LINEAR_GAP;
     else if (abpt->gap_open1 > 0 && abpt->gap_open2 == 0) abpt->gap_mode = ABPOA_AFFINE_GAP;
     else abpt->gap_mode = ABPOA_CONVEX_GAP;
@@ -96,6 +103,12 @@ abpoa_para_t *abpoa_init_para(void) {
     abpt->gap_mode = ABPOA_CONVEX_GAP;
     abpt->zdrop = -1;     // disable zdrop
     abpt->end_bonus = -1; // disable end bonus
+
+    abpt->inc_path_score = 0;
+    abpt->sort_input_seq = 0;
+    abpt->put_gap_on_right = 0;
+    abpt->put_gap_at_end = 0;
+
     abpt->wb = ABPOA_EXTRA_B; // extra bandwidth
     abpt->wf = ABPOA_EXTRA_F; // extra bandwidth
 
@@ -108,6 +121,7 @@ abpoa_para_t *abpoa_init_para(void) {
     abpt->out_msa = 0;    // output msa
     abpt->cons_algrm = ABPOA_HB; // consensus calling algorithm
     abpt->max_n_cons = 1; // number of max. generated consensus sequence
+    abpt->sub_aln = 0;
     abpt->min_freq = MULTIP_MIN_FREQ; 
     abpt->use_read_ids = 0;
     abpt->incr_fn = NULL; // incrementally align seq to an existing graph
@@ -135,6 +149,7 @@ abpoa_para_t *abpoa_init_para(void) {
     abpt->progressive_poa = 0; // progressive partial order alignment
 
     abpt->verbose = ABPOA_NONE_VERBOSE;
+    abpt->batch_index = 0; // 0 means not in batch mode
 
     // abpt->simd_flag = simd_check();
 
@@ -145,7 +160,7 @@ void abpoa_post_set_para(abpoa_para_t *abpt) {
     abpoa_set_gap_mode(abpt);
     if (abpt->out_msa || abpt->out_gfa || abpt->max_n_cons > 1 || abpt->cons_algrm == ABPOA_MF) {
         abpt->use_read_ids = 1;
-        if (abpt->out_msa || abpt->out_gfa) set_65536_table();
+        if (abpt->out_msa || abpt->out_gfa || abpt->max_n_cons > 1) set_65536_table();
         if (abpt->max_n_cons > 1 || abpt->cons_algrm == ABPOA_MF) set_bit_table16();
     }
     if (abpt->align_mode == ABPOA_LOCAL_MODE) abpt->wb = -1;
@@ -200,6 +215,7 @@ int abpoa_anchor_poa(abpoa_t *ab, abpoa_para_t *abpt, uint8_t **seqs, int **weig
     // uint8_t *seq1;
     for (_i = 0; _i < n_seq; ++_i) {
         i = read_id_map[_i]; read_id = exist_n_seq + i; qlen = seq_lens[i]; whole_res.n_cigar = 0, whole_res.m_cigar = 0, whole_res.graph_cigar = 0;
+        // fprintf(stderr, "seq: # %d\n", i);
         if (abpt->verbose >= ABPOA_DEBUG_VERBOSE) fprintf(stderr, "seq: # %d\n", i);
         // seq-to-graph alignment and add alignment within each split window
         if (_i == 0) ai = 0; else ai = par_c[_i-1];
@@ -353,6 +369,26 @@ void abpoa_output(abpoa_t *ab, abpoa_para_t *abpt, FILE *out_fp) {
     if (abpt->out_pog) abpoa_dump_pog(ab, abpt);
 }
 
+// only sort new sequences
+void abpoa_sort_seq_by_length(abpoa_seq_t *abs, int exist_n_seq, int n_seq) {
+    int i, j;
+    // sort by length
+    abpoa_seq_t *tmp_seq = (abpoa_seq_t*)calloc(1, sizeof(abpoa_seq_t));
+    tmp_seq->n_seq = 1; abpoa_realloc_seq(tmp_seq);
+    for (i = 0; i < n_seq-1; ++i) {
+        for (j = i+1; j < n_seq; ++j) {
+            if (abs->seq[exist_n_seq+i].l < abs->seq[exist_n_seq+j].l) {
+                // swap i and j
+                abpoa_cpy_abs(tmp_seq, 0, abs, exist_n_seq+i);
+                abpoa_cpy_abs(abs, exist_n_seq+i, abs, exist_n_seq+j);
+                abpoa_cpy_abs(abs, exist_n_seq+j, tmp_seq, 0);
+            }
+        }
+    }
+    // free
+    abpoa_free_seq(tmp_seq);
+}
+
 // do msa for a set of input sequences
 // @function: 
 //    generate consensus sequence
@@ -380,7 +416,6 @@ int abpoa_msa(abpoa_t *ab, abpoa_para_t *abpt, int n_seq, char **seq_names, int
             abs->name[exist_n_seq+i].l = 0; abs->name[exist_n_seq+i].m = 0;
         }
     }
-
     // always reset graph before perform POA
     int max_len = 0;
     for (i = 0; i < n_seq; ++i) {
@@ -438,6 +473,8 @@ int abpoa_msa1(abpoa_t *ab, abpoa_para_t *abpt, char *read_fn, FILE *out_fp) {
     gzFile readfp = xzopen(read_fn, "r"); kseq_t *ks = kseq_init(readfp);
     int i, j, n_seq = abpoa_read_seq(abs, ks);
 
+    if (abpt->sort_input_seq) abpoa_sort_seq_by_length(abs, exist_n_seq, n_seq);
+
     // always reset graph before perform POA
     int max_len = 0;
     for (i = 0; i < abs->n_seq; ++i) {
