@@ -3,7 +3,8 @@
 #include <math.h>
 #include "abpoa_align.h"
 #include "abpoa_seq.h"
-#include "simd_abpoa_align.h"
+#include "abpoa_align_simd.h"
+#include "abpoa_simd.h"
 #include "kdq.h"
 
 KDQ_INIT(int)
@@ -19,14 +20,16 @@ void abpoa_set_graph_node(abpoa_graph_t *abg, int node_i) {
     abg->node[node_i].in_edge_n = 0; abg->node[node_i].in_edge_m = 0;
     abg->node[node_i].out_edge_n = 0; abg->node[node_i].out_edge_m = 0;
     abg->node[node_i].aligned_node_n = 0; abg->node[node_i].aligned_node_m = 0;
-    abg->node[node_i].n_read = 0; abg->node[node_i].m_read = 0; abg->node[node_i].read_weight = NULL;
+    abg->node[node_i].n_read = 0; abg->node[node_i].n_span_read = 0; abg->node[node_i].m_read = 0; abg->node[node_i].read_weight = NULL;
     abg->node[node_i].read_ids_n = 0;
 }
 
 void abpoa_free_node(abpoa_node_t *node, int n) {
     int i, j;
     for (i = 0; i < n; ++i) {
-        if (node[i].in_edge_m > 0) free(node[i].in_id);
+        if (node[i].in_edge_m > 0) {
+            free(node[i].in_id); free(node[i].in_edge_weight);
+        }
         if (node[i].out_edge_m > 0) {
             free(node[i].out_id); free(node[i].out_edge_weight);
             if (node[i].read_ids_n > 0) {
@@ -45,7 +48,9 @@ void abpoa_free_node(abpoa_node_t *node, int n) {
 // 0: in_edge, 1: out_edge
 abpoa_graph_t *abpoa_realloc_graph_edge(abpoa_graph_t *abg, int io, int id, int use_read_ids) {
     if (io == 0) {
-        _uni_realloc(abg->node[id].in_id, abg->node[id].in_edge_n, abg->node[id].in_edge_m, int);
+        int m = abg->node[id].in_edge_m;
+        _uni_realloc(abg->node[id].in_id, abg->node[id].in_edge_n, m, int);
+        _uni_realloc(abg->node[id].in_edge_weight, abg->node[id].in_edge_n, abg->node[id].in_edge_m, int);
     } else {
         int edge_m = abg->node[id].out_edge_m;
         if (edge_m <= 0) {
@@ -183,6 +188,36 @@ void abpoa_free(abpoa_t *ab) {
     free(ab);
 }
 
+// sort in_id/out_id by edge weight in descending order
+void abpoa_sort_in_out_ids(abpoa_graph_t *abg) {
+    int i, j, k;
+    int tmp; uint64_t *tmp_read_ids;
+    for (i = 0; i < abg->node_n; ++i) {
+        // in_id
+        for (j = 0; j < abg->node[i].in_edge_n-1; ++j) {
+            for (k = j+1; k < abg->node[i].in_edge_n; ++k) {
+                if (abg->node[i].in_edge_weight[j] < abg->node[i].in_edge_weight[k]) {
+                    tmp = abg->node[i].in_id[j]; abg->node[i].in_id[j] = abg->node[i].in_id[k]; abg->node[i].in_id[k] = tmp;
+                    tmp = abg->node[i].in_edge_weight[j]; abg->node[i].in_edge_weight[j] = abg->node[i].in_edge_weight[k]; abg->node[i].in_edge_weight[k] = tmp;
+                }
+            }
+        }
+        // out_id
+        for (j = 0; j < abg->node[i].out_edge_n-1; ++j) {
+            for (k = j+1; k < abg->node[i].out_edge_n; ++k) {
+                if (abg->node[i].out_edge_weight[j] < abg->node[i].out_edge_weight[k]) {
+                    tmp = abg->node[i].out_id[j]; abg->node[i].out_id[j] = abg->node[i].out_id[k]; abg->node[i].out_id[k] = tmp;
+                    tmp = abg->node[i].out_edge_weight[j]; abg->node[i].out_edge_weight[j] = abg->node[i].out_edge_weight[k]; abg->node[i].out_edge_weight[k] = tmp;
+                    if (abg->node[i].read_ids_n > 0) {
+                        tmp_read_ids = abg->node[i].read_ids[j]; abg->node[i].read_ids[j] = abg->node[i].read_ids[k]; abg->node[i].read_ids[k] = tmp_read_ids;
+                    }
+                }
+            }
+        }
+    }
+
+}
+
 void abpoa_BFS_set_node_index(abpoa_graph_t *abg, int src_id, int sink_id) {
     int *id, cur_id, out_id, aligned_id;
     int index = 0, q_size, new_q_size;
@@ -273,6 +308,14 @@ void abpoa_BFS_set_node_remain(abpoa_graph_t *abg, int src_id, int sink_id) {
     err_fatal_simple("Failed to set node remain.");
 }
 
+// TODO 
+void abpoa_merge_nodes(abpoa_graph_t *abg, int src_id, int sink_id) {
+    if (abg->node_n <= 0) {
+        err_func_format_printf(__func__, "Empty graph.\n");
+        return;
+    }
+}
+
 // 1. index_to_node_id
 // 2. node_id_to_index
 // 3. node_id_to_rank
@@ -287,7 +330,7 @@ void abpoa_topological_sort(abpoa_graph_t *abg, abpoa_para_t *abpt) {
         // fprintf(stderr, "node_n: %d, index_rank_m: %d\n", node_n, abg->index_rank_m);
         abg->index_to_node_id = (int*)_err_realloc(abg->index_to_node_id, abg->index_rank_m * sizeof(int));
         abg->node_id_to_index = (int*)_err_realloc(abg->node_id_to_index, abg->index_rank_m * sizeof(int));
-        if (abpt->out_msa || abpt->max_n_cons > 1 || abpt->cons_algrm == ABPOA_MF) 
+        if (abpt->out_msa || abpt->max_n_cons > 1 || abpt->cons_algrm == ABPOA_MF)
             abg->node_id_to_msa_rank = (int*)_err_realloc(abg->node_id_to_msa_rank, abg->index_rank_m * sizeof(int));
         if (abpt->wb >= 0) {
             abg->node_id_to_max_pos_left = (int*)_err_realloc(abg->node_id_to_max_pos_left, abg->index_rank_m * sizeof(int));
@@ -299,6 +342,7 @@ void abpoa_topological_sort(abpoa_graph_t *abg, abpoa_para_t *abpt) {
     }
     // start from ABPOA_SRC_NODE_ID to ABPOA_SINK_NODE_ID
     abpoa_BFS_set_node_index(abg, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID);
+    abpoa_sort_in_out_ids(abg);
     // init min/max rank
     if (abpt->wb >= 0) {
         int i;
@@ -374,6 +418,24 @@ void abpoa_set_msa_rank(abpoa_graph_t *abg, int src_id, int sink_id) {
     }
 }
 
+int abpoa_get_node_weight(abpoa_graph_t *abg, int node_id) {
+    int i, weight = 0;
+    for (i = 0; i < abg->node[node_id].out_edge_n; ++i) {
+        weight += abg->node[node_id].out_edge_weight[i];
+    }
+    return weight;
+}
+
+int abpoa_get_incre_path_score(abpoa_graph_t *abg, int node_id, int in_id_idx) {
+    if (in_id_idx < 0 || in_id_idx >= abg->node[node_id].in_edge_n) err_fatal(__func__, "Unexpected in_id_idx: %d.", in_id_idx);
+    int pre_node_id = abg->node[node_id].in_id[in_id_idx];
+    int node_w = abpoa_get_node_weight(abg, pre_node_id);
+    int edge_w = abg->node[node_id].in_edge_weight[in_id_idx];
+    if (node_w == 0 || edge_w == 0) return 0;
+    int score = round(log((double)edge_w/(double)node_w));
+    return MAX_OF_TWO(score, -20); // -20 is the minimum score
+}
+
 int abpoa_get_aligned_id(abpoa_graph_t *abg, int node_id, uint8_t base) {
     int i, aln_id;
     abpoa_node_t *node = abg->node;
@@ -424,6 +486,15 @@ int abpoa_add_graph_edge(abpoa_graph_t *abg, int from_id, int to_id, int check_e
     int out_edge_i = -1;
     if (check_edge) {
         int i;
+        // in edge
+        for (i = 0; i < abg->node[to_id].in_edge_n; ++i) {
+            if (abg->node[to_id].in_id[i] == from_id) { // edge exists
+                abg->node[to_id].in_edge_weight[i] += w; // update weight on existing edge
+                // update label id
+                break;
+            }
+        }
+        // out edge
         for (i = 0; i < out_edge_n; ++i) {
             if (abg->node[from_id].out_id[i] == to_id) { // edge exists
                 abg->node[from_id].out_edge_weight[i] += w; // update weight on existing edge
@@ -440,6 +511,7 @@ int abpoa_add_graph_edge(abpoa_graph_t *abg, int from_id, int to_id, int check_e
         /// in edge
         abpoa_realloc_graph_edge(abg, 0, to_id, 0);
         abg->node[to_id].in_id[abg->node[to_id].in_edge_n] = from_id;
+        abg->node[to_id].in_edge_weight[abg->node[to_id].in_edge_n] = w;
         ++abg->node[to_id].in_edge_n;
         /// out edge
         abpoa_realloc_graph_edge(abg, 1, from_id, add_read_id);
@@ -483,7 +555,22 @@ int abpoa_add_graph_edge(abpoa_graph_t *abg, int from_id, int to_id, int check_e
     return ret;
 }
 
-void abpoa_add_graph_sequence(abpoa_graph_t *abg, uint8_t *seq, int *weight, int seq_l, int *qpos_to_node_id, int start, int end, uint8_t add_read_id, uint8_t add_read_weight, int read_id, int read_ids_n, int tot_read_n) {
+// XXX n_span_read should be updated after adding all edges/nodes in the graph, right before topological sort/consensus calling
+void abpoa_update_node_n_span_reads(abpoa_graph_t *abg, int src_id, int sink_id, int inc_both_ends) {
+    int src_index = abg->node_id_to_index[src_id];
+    int sink_index = abg->node_id_to_index[sink_id];
+    int i;
+    for (i = src_index+1; i < sink_index; ++i) {
+        int id = abg->index_to_node_id[i];
+        abg->node[id].n_span_read += 1;
+    }
+    if (inc_both_ends) {
+        abg->node[src_id].n_span_read += 1;
+        abg->node[sink_id].n_span_read += 1;
+    }
+}
+
+void abpoa_add_graph_sequence(abpoa_graph_t *abg, abpoa_para_t *abpt, uint8_t *seq, int *weight, int seq_l, int *qpos_to_node_id, int start, int end, uint8_t add_read_id, uint8_t add_read_weight, int read_id, int read_ids_n, int tot_read_n) {
     if (start >= seq_l || end <= start) err_fatal(__func__, "seq_l: %d\tstart: %d\tend: %d.", seq_l, start, end);
     if (end > seq_l) end = seq_l;
 
@@ -493,21 +580,26 @@ void abpoa_add_graph_sequence(abpoa_graph_t *abg, uint8_t *seq, int *weight, int
         cur_node_id = abpoa_add_graph_node(abg, seq[i]);
         if (qpos_to_node_id) qpos_to_node_id[i] = cur_node_id;
         abpoa_add_graph_edge(abg, last_node_id, cur_node_id, 0, weight[i], add_read_id, add_read_weight, read_id, read_ids_n, tot_read_n);
+        abg->node[cur_node_id].n_span_read = abg->node[last_node_id].n_span_read;
         last_node_id = cur_node_id;
     }
 
     abpoa_add_graph_edge(abg, last_node_id, ABPOA_SINK_NODE_ID, 0, weight[seq_l-1], add_read_id, add_read_weight, read_id, read_ids_n, tot_read_n);
     abg->is_called_cons = abg->is_set_msa_rank = abg->is_topological_sorted = 0;
-    // abpoa_topological_sort(abg, abpt);
+    abpoa_topological_sort(abg, abpt);
+    abpoa_update_node_n_span_reads(abg, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID, 1);
+    // abpoa_merge_nodes(abg, ABPOA_SRC_NODE_ID, ABPOA_SINK_NODE_ID);
 }
 
-int is_full_upstream_subgraph(abpoa_graph_t *abg, int up_index, int down_index) {
+int is_full_upstream_subgraph(abpoa_graph_t *abg, int up_index, int down_index, int beg_index, int end_index) {
     int i, j, id, in_id;
+    int min_index = MIN_OF_TWO(up_index, beg_index);
+    int max_index = MAX_OF_TWO(down_index, end_index);
     for (i = up_index+1; i <= down_index; ++i) {
         id = abg->index_to_node_id[i];
         for (j = 0; j < abg->node[id].in_edge_n; ++j) {
             in_id = abg->node[id].in_id[j];
-            if (abg->node_id_to_index[in_id] < up_index) return 0;
+            if (abg->node_id_to_index[in_id] < min_index || abg->node_id_to_index[in_id] > max_index) return 0;
         }
     }
     return 1;
@@ -526,7 +618,7 @@ int abpoa_upstream_index(abpoa_graph_t *abg, int beg_index, int end_index) {
                 min_index = MIN_OF_TWO(min_index, in_index);
             }
         }
-        if (is_full_upstream_subgraph(abg, min_index, beg_index)) {
+        if (is_full_upstream_subgraph(abg, min_index, beg_index, beg_index, end_index)) {
             return min_index;
         } else {
             end_index = beg_index;
@@ -560,7 +652,7 @@ int abpoa_downstream_index(abpoa_graph_t *abg, int beg_index, int end_index) {
                 max_index = MAX_OF_TWO(max_index, out_index);
             }
         }
-        if (is_full_upstream_subgraph(abg, end_index, max_index)) {
+        if (is_full_upstream_subgraph(abg, end_index, max_index, beg_index, end_index)) {
             return max_index;
         } else {
             beg_index = end_index;
@@ -605,7 +697,7 @@ int abpoa_add_subgraph_alignment(abpoa_t *ab, abpoa_para_t *abpt, int beg_node_i
     } else weight = _weight;
 
     if (abg->node_n == 2) { // empty graph
-        abpoa_add_graph_sequence(abg, seq, weight, seq_l, qpos_to_node_id, 0, seq_l, add_read_id, add_read_weight, read_id, read_ids_n, tot_read_n);
+        abpoa_add_graph_sequence(abg, abpt, seq, weight, seq_l, qpos_to_node_id, 0, seq_l, add_read_id, add_read_weight, read_id, read_ids_n, tot_read_n);
         if (_weight == NULL) free(weight);
         return 0;
     } else {
@@ -632,11 +724,14 @@ int abpoa_add_subgraph_alignment(abpoa_t *ab, abpoa_para_t *abpt, int beg_node_i
                 if ((aligned_id = abpoa_get_aligned_id(abg, node_id, seq[query_id])) != -1) {
                     if (last_id != beg_node_id || inc_both_ends) add = 1; else add = 0;
                     abpoa_add_graph_edge(abg, last_id, aligned_id, 1-last_new, weight[query_id], add_read_id&add, add_read_weight, read_id, read_ids_n, tot_read_n);
+                    if (add == 0) abg->node[last_id].n_read--;
                     last_id = aligned_id; last_new = 0;
                 } else {
                     new_id = abpoa_add_graph_node(abg, seq[query_id]);
                     if (last_id != beg_node_id || inc_both_ends) add = 1; else add = 0;
                     abpoa_add_graph_edge(abg, last_id, new_id, 0, weight[query_id], add_read_id&add, add_read_weight, read_id, read_ids_n, tot_read_n);
+                    abg->node[new_id].n_span_read = abg->node[last_id].n_span_read;
+                    if (add == 0) abg->node[last_id].n_read--;
                     last_id = new_id; last_new = 1;
                     // add new_id to node_id's aligned node
                     abpoa_add_graph_aligned_node(abg, node_id, new_id);
@@ -644,6 +739,7 @@ int abpoa_add_subgraph_alignment(abpoa_t *ab, abpoa_para_t *abpt, int beg_node_i
             } else { // match
                 if (last_id != beg_node_id || inc_both_ends) add = 1; else add = 0;
                 abpoa_add_graph_edge(abg, last_id, node_id, 1-last_new, weight[query_id], add_read_id&add, add_read_weight, read_id, read_ids_n, tot_read_n);
+                if (add == 0) abg->node[last_id].n_read--;
                 last_id = node_id; last_new = 0;
             }
             if (qpos_to_node_id) qpos_to_node_id[query_id] = last_id;
@@ -654,6 +750,8 @@ int abpoa_add_subgraph_alignment(abpoa_t *ab, abpoa_para_t *abpt, int beg_node_i
                 new_id = abpoa_add_graph_node(abg, seq[query_id-j]);
                 if (last_id != beg_node_id || inc_both_ends) add = 1; else add = 0;
                 abpoa_add_graph_edge(abg, last_id, new_id, 0, weight[query_id-j], add_read_id&add, add_read_weight, read_id, read_ids_n, tot_read_n);
+                abg->node[new_id].n_span_read = abg->node[last_id].n_span_read;
+                if (add == 0) abg->node[last_id].n_read--;
                 last_id = new_id; last_new = 1;
                 if (qpos_to_node_id) qpos_to_node_id[query_id-j] = last_id;
             }
@@ -665,8 +763,11 @@ int abpoa_add_subgraph_alignment(abpoa_t *ab, abpoa_para_t *abpt, int beg_node_i
     // if (inc_both_ends) add = 1; else add = 0; XXX end_node_id is always excluded when adding weight
     // abpoa_add_graph_edge(abg, last_id, end_node_id, 1-last_new, w, add_read_id&add, read_id, read_ids_n);
     abpoa_add_graph_edge(abg, last_id, end_node_id, 1-last_new, weight[seq_l-1], add_read_id, add_read_weight, read_id, read_ids_n, tot_read_n);
-    abg->is_called_cons = abg->is_topological_sorted = 0;
-    // abpoa_topological_sort(abg, abpt);
+    abg->is_called_cons = abg->is_set_msa_rank = abg->is_topological_sorted = 0;
+    abpoa_topological_sort(abg, abpt);
+    // update node.n_read
+    abpoa_update_node_n_span_reads(abg, beg_node_id, end_node_id, inc_both_ends);
+    // abpoa_merge_nodes(abg, beg_node_id, end_node_id);
     if (_weight == NULL) free(weight);
     return 0;
 }
@@ -687,7 +788,7 @@ void abpoa_reset(abpoa_t *ab, abpoa_para_t *abpt, int qlen) {
             for (k = 0; k < abg->node[i].read_ids_n; ++k) abg->node[i].read_ids[j][k] = 0;
         }
         abg->node[i].in_edge_n = abg->node[i].out_edge_n = abg->node[i].aligned_node_n = 0;
-        abg->node[i].n_read = 0;
+        abg->node[i].n_read = 0; abg->node[i].n_span_read = 0;
             
     }
     abg->node_n = 2;
